@@ -16,11 +16,9 @@
  *    Phone: 1-703-847-0040 ext 103
  */
 
-static char *version = 
+static const char *version = 
 	"Equalizer: $Revision: 3.12 $ $Date: 1995/01/19 $ Simon Janes (simon@ncm.com)\n";
 
-#include <linux/config.h>
-
 /*
  * Sources:
  *   skeleton.c by Donald Becker.
@@ -104,6 +102,12 @@ static char *version =
  *
  */
 
+#ifdef MODULE
+#include <linux/module.h>
+#include <linux/version.h>
+#endif
+
+#include <linux/config.h>
 #include <linux/kernel.h>
 #include <linux/sched.h>
 #include <linux/types.h>
@@ -122,6 +126,7 @@ static char *version =
 
 #include <linux/netdevice.h>
 #include <linux/if.h>
+#include <linux/if_arp.h>
 #include <linux/timer.h>
 
 #include <linux/if_eql.h>
@@ -214,10 +219,17 @@ eql_init(struct device *dev)
 
   /* Initialize the device structure. */
   dev->priv = kmalloc (sizeof (equalizer_t), GFP_KERNEL);
+  if (dev->priv == NULL)
+      return -ENOMEM;
   memset (dev->priv, 0, sizeof (equalizer_t));
   eql = (equalizer_t *) dev->priv;
 
   eql->stats = kmalloc (sizeof (struct enet_statistics), GFP_KERNEL);
+  if (eql->stats == NULL) {
+      kfree(dev->priv);
+      dev->priv = NULL;
+      return -ENOMEM;
+  }
   memset (eql->stats, 0, sizeof (struct enet_statistics));
 
   init_timer (&eql->timer);
@@ -283,6 +295,9 @@ eql_open(struct device *dev)
 	eql->timer_on = 1;
 	add_timer (&eql->timer);
 
+#ifdef MODULE
+	MOD_INC_USE_COUNT;
+#endif
 	return 0;
       }
   return 1;
@@ -307,6 +322,10 @@ eql_close(struct device *dev)
 
   eql_delete_slave_queue (eql->queue);
 
+#ifdef MODULE
+  MOD_DEC_USE_COUNT;
+#endif
+
   return 0;
 }
 
@@ -749,6 +768,8 @@ eql_new_slave_queue(struct device *dev)
   slave_t *tail_slave;
 
   queue = (slave_queue_t *) kmalloc (sizeof (slave_queue_t), GFP_KERNEL);
+  if (queue == NULL)
+    return 0;
   memset (queue, 0, sizeof (slave_queue_t));
 
   head_slave = eql_new_slave ();
@@ -765,6 +786,10 @@ eql_new_slave_queue(struct device *dev)
     }
   else
     {
+      if (head_slave)
+         kfree(head_slave);
+      if (tail_slave)
+         kfree(tail_slave);
       kfree (queue);
       return 0;
     }
@@ -847,6 +872,7 @@ eql_remove_slave(slave_queue_t *queue, slave_t *slave)
     {
       prev->next = current->next;
       queue->num_slaves--;
+      sti();
       return current;
     }
 
@@ -1165,6 +1191,32 @@ eql_timer(unsigned long param)
     }
 }
 
+#ifdef MODULE
+char kernel_version[] = UTS_RELEASE;
+static struct device dev_eql = {
+	"eql", 0, 0, 0, 0, 0, 0, 0, 0, 0, NULL, eql_init };
+
+int init_module(void)
+{
+	if (register_netdev(&dev_eql) != 0) {
+		printk("eql: register_netdev() returned non-zero.\n");
+		return -EIO;
+	}
+	return 0;
+}
+
+void
+cleanup_module(void)
+{
+	if (MOD_IN_USE)
+		printk("eql: device busy, remove delayed\n");
+	else
+	{
+		unregister_netdev(&dev_eql);
+	}
+}
+#endif /* MODULE */
+
 /*
  * Local Variables: 
  * compile-command: "gcc -D__KERNEL__ -I/usr/src/linux/net/inet -Wall -Wstrict-prototypes -O6 -m486 -c eql.c"
