@@ -106,7 +106,7 @@ struct thread_struct {
 	unsigned long v86flags, v86mask, v86mode;
 };
 
-#define INIT_MMAP { &init_task, 0, 0x40000000, PAGE_SHARED, VM_READ | VM_WRITE | VM_EXEC }
+#define INIT_MMAP { &init_mm, 0, 0x40000000, PAGE_SHARED, VM_READ | VM_WRITE | VM_EXEC }
 
 #define INIT_TSS  { \
 	0,0, \
@@ -132,4 +132,12 @@ static inline void start_thread(struct pt_regs * regs, unsigned long eip, unsign
 	regs->esp = esp;
 }
 
+/*
+ * Return saved PC of a blocked thread.
+ */
+extern inline unsigned long thread_saved_pc(struct thread_struct *t)
+{
+	return ((unsigned long *)t->esp)[3];
+}
+
 #endif /* __ASM_I386_PROCESSOR_H */
