@@ -3,11 +3,16 @@
 
 #include <linux/config.h>
 #include <linux/fs.h>
+#include <linux/malloc.h>
 
 /*
  * The proc filesystem constants/structures
  */
 
+/*
+ * We always define these enumerators
+ */
+
 enum root_directory_inos {
 	PROC_ROOT_INO = 1,
 	PROC_LOADAVG,
@@ -20,9 +25,7 @@ enum root_directory_inos {
 	PROC_SELF,	/* will change inode # */
 	PROC_NET,
         PROC_SCSI,
-#ifdef CONFIG_DEBUG_MALLOC
 	PROC_MALLOC,
-#endif
 	PROC_KCORE,
 	PROC_MODULES,
 	PROC_STAT,
@@ -55,7 +58,6 @@ enum pid_subdirectory_inos {
 
 enum net_directory_inos {
 	PROC_NET_UNIX = 128,
-#ifdef CONFIG_INET
 	PROC_NET_ARP,
 	PROC_NET_ROUTE,
 	PROC_NET_DEV,
@@ -63,46 +65,25 @@ enum net_directory_inos {
 	PROC_NET_TCP,
 	PROC_NET_UDP,
 	PROC_NET_SNMP,
-#ifdef CONFIG_INET_RARP
 	PROC_NET_RARP,
-#endif
-#ifdef CONFIG_IP_MULTICAST
 	PROC_NET_IGMP,
-#endif
-#ifdef CONFIG_IP_FIREWALL
 	PROC_NET_IPFWFWD,
 	PROC_NET_IPFWBLK,
-#endif
-#ifdef CONFIG_IP_ACCT
 	PROC_NET_IPACCT,
-#endif
-#ifdef CONFIG_IP_MASQUERADE
 	PROC_NET_IPMSQHST,
-#endif
-#if	defined(CONFIG_WAVELAN)
 	PROC_NET_WAVELAN,
-#endif	/* defined(CONFIG_WAVELAN) */
-#endif
-#ifdef CONFIG_IPX
 	PROC_NET_IPX_INTERFACE,
 	PROC_NET_IPX_ROUTE,
 	PROC_NET_IPX,
-#endif
-#ifdef CONFIG_ATALK
 	PROC_NET_ATALK,
 	PROC_NET_AT_ROUTE,
 	PROC_NET_ATIF,
-#endif
-#ifdef CONFIG_AX25
 	PROC_NET_AX25_ROUTE,
 	PROC_NET_AX25,
 	PROC_NET_AX25_CALLS,
-#ifdef CONFIG_NETROM
 	PROC_NET_NR_NODES,
 	PROC_NET_NR_NEIGH,
 	PROC_NET_NR,
-#endif
-#endif
 	PROC_NET_SOCKSTAT,
 	PROC_NET_LAST
 };
@@ -117,7 +98,7 @@ enum scsi_directory_inos {
 	PROC_SCSI_AIC7XXX,
 	PROC_SCSI_BUSLOGIC,
 	PROC_SCSI_U14_34F,
-	PROC_SCSI_FUTURE_DOMAIN,
+	PROC_SCSI_FDOMAIN,
 	PROC_SCSI_GENERIC_NCR5380,
 	PROC_SCSI_IN2000,
 	PROC_SCSI_PAS16,
@@ -136,23 +117,113 @@ enum scsi_directory_inos {
 
 #define PROC_SUPER_MAGIC 0x9fa0
 
+/*
+ * This is not completely implemented yet. The idea is to
+ * create a in-memory tree (like the actual /proc filesystem
+ * tree) of these proc_dir_entries, so that we can dynamically
+ * add new files to /proc.
+ *
+ * The "next" pointer creates a linked list of one /proc directory,
+ * while parent/subdir create the directory structure (every
+ * /proc file has a parent, but "subdir" is NULL for all
+ * non-directory entries).
+ *
+ * "get_info" is called at "read", while "fill_inode" is used to
+ * fill in file type/protection/owner information specific to the
+ * particular /proc file.
+ */
 struct proc_dir_entry {
 	unsigned short low_ino;
 	unsigned short namelen;
-	char * name;
+	const char *name;
+	mode_t mode;
+	nlink_t nlink;
+	uid_t uid;
+	gid_t gid;
+	unsigned long size;
+	struct inode_operations * ops;
+	int (*get_info)(char *, char **, off_t, int, int);
+	void (*fill_inode)(struct inode *);
+	struct proc_dir_entry *next, *parent, *subdir;
 };
 
+extern struct proc_dir_entry proc_root;
+extern struct proc_dir_entry proc_net;
+extern struct proc_dir_entry proc_scsi;
+extern struct proc_dir_entry proc_pid;
+extern struct proc_dir_entry proc_pid_fd;
+
+extern struct inode_operations proc_scsi_inode_operations;
+
+extern void proc_root_init(void);
+extern void proc_base_init(void);
+extern void proc_net_init(void);
+
+extern int proc_register(struct proc_dir_entry *, struct proc_dir_entry *);
+extern int proc_unregister(struct proc_dir_entry *, int);
+
+static inline int proc_net_register(struct proc_dir_entry * x)
+{
+	return proc_register(&proc_net, x);
+}
+
+static inline int proc_net_unregister(int x)
+{
+	return proc_unregister(&proc_net, x);
+}
+
+static inline int proc_scsi_register(struct proc_dir_entry *driver, 
+				     struct proc_dir_entry *x)
+{
+    x->ops = &proc_scsi_inode_operations;
+    if(x->low_ino < PROC_SCSI_FILE){
+	return(proc_register(&proc_scsi, x));
+    }else{
+	return(proc_register(driver, x));
+    }
+}
+
+static inline int proc_scsi_unregister(struct proc_dir_entry *driver, int x)
+{
+    extern void scsi_init_free(char *ptr, unsigned int size);
+
+    if(x <= PROC_SCSI_FILE)
+	return(proc_unregister(&proc_scsi, x));
+    else {
+	struct proc_dir_entry **p = &driver->subdir, *dp;
+	int ret;
+
+	while ((dp = *p) != NULL) {
+		if (dp->low_ino == x) 
+		    break;
+		p = &dp->next;
+	}
+	ret = proc_unregister(driver, x);
+	scsi_init_free((char *) dp, sizeof(struct proc_dir_entry) + 4);
+	return(ret);
+    }
+}
+
 extern struct super_block *proc_read_super(struct super_block *,void *,int);
-extern void proc_put_inode(struct inode *);
-extern void proc_put_super(struct super_block *);
+extern struct inode * proc_get_inode(struct super_block *, int, struct proc_dir_entry *);
 extern void proc_statfs(struct super_block *, struct statfs *, int);
 extern void proc_read_inode(struct inode *);
 extern void proc_write_inode(struct inode *);
 extern int proc_match(int, const char *, struct proc_dir_entry *);
 
-extern struct inode_operations proc_root_inode_operations;
+/*
+ * These are generic /proc routines that use the internal
+ * "struct proc_dir_entry" tree to traverse the filesystem.
+ *
+ * The /proc root directory has extended versions to take care
+ * of the /proc/<pid> subdirectories.
+ */
+extern int proc_readdir(struct inode *, struct file *, void *, filldir_t);
+extern int proc_lookup(struct inode *, const char *, int, struct inode **);
+
 extern struct inode_operations proc_base_inode_operations;
 extern struct inode_operations proc_net_inode_operations;
+extern struct inode_operations proc_netdir_inode_operations;
 extern struct inode_operations proc_scsi_inode_operations;
 extern struct inode_operations proc_mem_inode_operations;
 extern struct inode_operations proc_array_inode_operations;
