@@ -29,8 +29,6 @@
 
 #include <asm/bugs.h>
 
-extern unsigned long * prof_buffer;
-extern unsigned long prof_len;
 extern char _stext, _etext;
 extern char *linux_banner;
 
@@ -64,6 +62,7 @@ extern void aha152x_setup(char *str, int *ints);
 extern void aha1542_setup(char *str, int *ints);
 extern void aic7xxx_setup(char *str, int *ints);
 extern void buslogic_setup(char *str, int *ints);
+extern void fdomain_setup(char *str, int *ints);
 extern void scsi_luns_setup(char *str, int *ints);
 extern void sound_setup(char *str, int *ints);
 #ifdef CONFIG_CDU31A
@@ -144,11 +143,24 @@ char *get_options(char *str, int *ints)
 	return(cur);
 }
 
+static void profile_setup(char *str, int *ints)
+{
+	if (ints[0] > 0)
+		prof_shift = (unsigned long) ints[1];
+	else
+#ifdef CONFIG_PROFILE_SHIFT
+		prof_shift = CONFIG_PROFILE_SHIFT;
+#else
+		prof_shift = 2;
+#endif
+}
+
 struct {
 	const char *str;
 	void (*setup_func)(char *, int *);
 } bootsetups[] = {
 	{ "reserve=", reserve_setup },
+	{ "profile=", profile_setup },
 	{ "ramdisk=", ramdisk_setup },
 #ifdef CONFIG_BUGi386
 	{ "no-hlt", no_halt },
@@ -160,13 +172,7 @@ struct {
 #ifdef CONFIG_SCSI
 	{ "max_scsi_luns=", scsi_luns_setup },
 #endif
-#ifdef CONFIG_BLK_DEV_IDE
-	{ "hda=", hda_setup },
-	{ "hdb=", hdb_setup },
-	{ "hdc=", hdc_setup },
-	{ "hdd=", hdd_setup },
-	{ "hd=",  ide_setup },
-#elif defined(CONFIG_BLK_DEV_HD)
+#if defined(CONFIG_BLK_DEV_HD)
 	{ "hd=", hd_setup },
 #endif
 #ifdef CONFIG_CHR_DEV_ST
@@ -200,6 +206,9 @@ struct {
 #ifdef CONFIG_SCSI_BUSLOGIC
 	{ "buslogic=", buslogic_setup},
 #endif
+#ifdef CONFIG_SCSI_FUTURE_DOMAIN
+	{ "fdomain=", fdomain_setup},
+#endif
 #ifdef CONFIG_BLK_DEV_XD
 	{ "xd=", xd_setup },
 #endif
@@ -253,6 +262,13 @@ static int checksetup(char *line)
 	int i = 0;
 	int ints[11];
 
+#ifdef CONFIG_BLK_DEV_IDE
+	/* ide driver needs the basic string, rather than pre-processed values */
+	if (!strncmp(line,"ide",3) || (!strncmp(line,"hd",2) && line[2] != '=')) {
+		ide_setup(line);
+		return 1;
+	}
+#endif
 	while (bootsetups[i].str) {
 		int n = strlen(bootsetups[i].str);
 		if (!strncmp(line,bootsetups[i].str,n)) {
@@ -264,11 +280,20 @@ static int checksetup(char *line)
 	return 0;
 }
 
-unsigned long loops_per_sec = 1;
+/* this should be approx 2 Bo*oMips to start (note initial shift), and will
+   still work even if initally too large, it will just take slightly longer */
+unsigned long loops_per_sec = (1<<12);
+
+/* This is the number of bits of precision for the loops_per_second.  Each
+   bit takes on average 1.5/HZ seconds.  This (like the original) is a little
+   better than 1% */
+#define LPS_PREC 8
 
 static void calibrate_delay(void)
 {
 	int ticks;
+	int loopbit;
+	int lps_precision = LPS_PREC;
 
 	printk("Calibrating delay loop.. ");
 	while (loops_per_sec <<= 1) {
@@ -280,17 +305,31 @@ static void calibrate_delay(void)
 		ticks = jiffies;
 		__delay(loops_per_sec);
 		ticks = jiffies - ticks;
-		if (ticks >= HZ) {
-			loops_per_sec = muldiv(loops_per_sec, HZ, ticks);
-			printk("ok - %lu.%02lu BogoMips\n",
-				loops_per_sec/500000,
-				(loops_per_sec/5000) % 100);
-			return;
+		if (ticks)
+			break;
 		}
-	}
-	printk("failed\n");
-}
 
+/* Do a binary approximation to get loops_per_second set to equal one clock
+   (up to lps_precision bits) */
+	loops_per_sec >>= 1;
+	loopbit = loops_per_sec;
+	while ( lps_precision-- && (loopbit >>= 1) ) {
+		loops_per_sec |= loopbit;
+		ticks = jiffies;
+		while (ticks == jiffies);
+		ticks = jiffies;
+		__delay(loops_per_sec);
+		if (jiffies != ticks)	/* longer than 1 tick */
+			loops_per_sec &= ~loopbit;
+	}
+
+/* finally, adjust loops per second in terms of seconds instead of clocks */	
+	loops_per_sec *= HZ;
+/* Round the value and print it */	
+	printk("ok - %lu.%02lu BogoMIPS\n",
+		(loops_per_sec+2500)/500000,
+		((loops_per_sec+2500)/5000) % 100);
+}
 
 /*
  * This is a simple kernel command line parsing function: it parses
@@ -324,14 +363,16 @@ static void parse_options(char *line)
 			int n;
 			line += 5;
 			if (strncmp(line,"/dev/",5)) {
-				ROOT_DEV = simple_strtoul(line,NULL,16);
+				ROOT_DEV = to_kdev_t(
+					     simple_strtoul(line,NULL,16));
 				continue;
 			}
 			line += 5;
 			for (n = 0 ; devnames[n] ; n++) {
 				int len = strlen(devnames[n]);
 				if (!strncmp(line,devnames[n],len)) {
-					ROOT_DEV = devnums[n]+simple_strtoul(line+len,NULL,0);
+					ROOT_DEV = to_kdev_t(devnums[n]+
+					     simple_strtoul(line+len,NULL,0));
 					break;
 				}
 			}
@@ -369,9 +410,10 @@ static void parse_options(char *line)
 	envp_init[envs+1] = NULL;
 }
 
-extern void check_bugs(void);
 extern void setup_arch(char **, unsigned long *, unsigned long *);
 
+static char init_stack[PAGE_SIZE];
+
 asmlinkage void start_kernel(void)
 {
 	char * command_line;
@@ -387,12 +429,20 @@ asmlinkage void start_kernel(void)
 	parse_options(command_line);
 	init_modules();
 #ifdef CONFIG_PROFILE
-	prof_buffer = (unsigned long *) memory_start;
-	/* only text is profiled */
-	prof_len = (unsigned long) &_etext - (unsigned long) &_stext;
-	prof_len >>= CONFIG_PROFILE_SHIFT;
-	memory_start += prof_len * sizeof(unsigned long);
+	if (!prof_shift)
+#ifdef CONFIG_PROFILE_SHIFT
+		prof_shift = CONFIG_PROFILE_SHIFT;
+#else
+		prof_shift = 2;
 #endif
+#endif
+	if (prof_shift) {
+		prof_buffer = (unsigned long *) memory_start;
+		/* only text is profiled */
+		prof_len = (unsigned long) &_etext - (unsigned long) &_stext;
+		prof_len >>= prof_shift;
+		memory_start += prof_len * sizeof(unsigned long);
+	}
 	memory_start = console_init(memory_start,memory_end);
 #ifdef CONFIG_PCI
 	memory_start = pci_init(memory_start,memory_end);
@@ -424,7 +474,8 @@ asmlinkage void start_kernel(void)
 
 	printk(linux_banner);
 
-	if (!fork())		/* we count on this going ok */
+	/* we count on the clone going ok */
+	if (!clone(CLONE_VM, init_stack+sizeof(init_stack)))
 		init();
 /*
  * task[0] is meant to be used as an "idle" task: it may not sleep, but
